﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading.Tasks;
using System.Management.Automation;
using System.Management.Automation.Runspaces;
using PSModuleSample;

namespace SampleGUIApplication
{
    public class CommandInvoker
    {
        public CommandInvoker(string cmdletName, Dictionary<string, object> parameters)
        {
            _cmdletName = cmdletName;
            _parameters = parameters;
        }

        private string _cmdletName;

        public string CmdletName
        {
            get { return _cmdletName; }
            set { _cmdletName = value; }
        }

        private Dictionary<string, object> _parameters;

        public Dictionary<string, object> Parameters
        {
            get { return _parameters; }
            set { _parameters = value; }
        }

        private DateTime _startTime;

        public DateTime StartTime
        {
            get { return _startTime; }
            private set { _startTime = value; }
        }

        private PSCommand _command;

        public List<T> Execute<T>()
        {
            InitialSessionState state = InitialSessionState.CreateDefault();
            state.ImportPSModule(new[]{"PSModuleSample"});
            List<T> results = null;
            using (Runspace rs = RunspaceFactory.CreateRunspace(state))
            {
                rs.Open();
                try
                {
                    using (PowerShell ps = PowerShell.Create())
                    {
                        _command = new PSCommand();
                        _command.AddCommand(_cmdletName);
                        _parameters.ToList().ForEach(x => _command.AddParameter(x.Key, x.Value));
                        ps.Commands = _command;
                        ps.Runspace = rs;
                        StartTime = DateTime.Now;
                        results = ps.Invoke().Select(x => (T)x.BaseObject).ToList<T>();
                    }
                }
                catch (Exception e)
                {
                    System.Windows.MessageBox.Show(e.Message);
                }
            }
            return results;
        }

        public string Script
        {
            get
            {
                var script = new StringBuilder();
                script.Append(_command.Commands[0].CommandText);

                if (_command.Commands[0].Parameters.Count > 0)
                {
                    _command.Commands[0].Parameters.ToList().ForEach((x) => {
                        if (x.Value is bool)  //SwitchParameter
                        {
                            script.Append(string.Format(" -{0}", x.Name));
                        }
                        else if (x.Value is string) //stringの場合はエスケープする
                        {
                            script.Append(string.Format(" -{0} \"{1}\"", x.Name, x.Value.ToString().Replace("\"","\"\"")));
                        }
                        else
                        {
                            script.Append(string.Format(" -{0} {1}", x.Name, x.Value));
                        }
                    });
                }
                return script.ToString();
            }
        }
    }

    public class InvokeHistory
    {
        private string _filePath;
        public InvokeHistory()
        {
            _filePath = string.Format(Path.Combine( Path.GetTempPath(),"PSModuleSampleLog_{0}.ps1"),DateTime.Now.ToString("yyyyMMddHHmmss"));
            File.AppendAllLines(_filePath, new[]{
                "Import-Module PSModuleSample",
                ""
            });
        }

        public void Write(CommandInvoker invoker)
        {
            File.AppendAllLines(_filePath,new[]{
                invoker.StartTime.ToString("# yyyy/MM/dd HH:mm:ss"),
                invoker.Script
            });
        }
    }
}
